"""
Sending SMS with letter.
"""
import twilio
from twilio.rest import TwilioRestClient

class TwillioPostie(object):
    """
    Render messages from templates and handle deliery.
    """
    def __init__(self, sid=None, token=None):
        self.client = TwilioRestClient(sid, token)

    def send(self, to, from_, body):
        """
        Send BODY to TO from FROM as an SMS!
        """
        try:
            msg = self.client.sms.messages.create(
                body=body,
                to=to,
                from_=from_
                )
            print msg.sid
        except twilio.TwilioRestException as e:
            raise


class SMS(object):
    """
    The top level API for our SMS messages
    """
    @classmethod
    def send(klass):
        klass.Postie.send(klass.To, klass.From, klass.Body)



"""
Sending messages on Social media services with letter.
"""
import collections

import tweepy

from letter import Error

__all__ = [
    'AccountNotFoundError',
    'TweetTooLongError',
    'TwitterAccount',
    'TwitterPostie',
    'Tweet'
    ]

class AccountNotFoundError(Error): pass
class TweetTooLongError(Error): pass
TwitterAccount = collections.namedtuple('TwitterAccount',
                                        'access_token access_token_secret')

class TwitterPostie(object):
    """
    Deliver messages via twitter
    """
    def __init__(self, consumer_key='', consumer_secret='', accounts={}):
        self.auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
        self.accounts = accounts

    def send(self, to, from_, body, dm=False):
        """
        Send BODY as an @message from FROM to TO

        If we don't have the access tokens for FROM, raise AccountNotFoundError.
        If the tweet resulting from '@{0} {1}'.format(TO, BODY) is > 140 chars
        raise TweetTooLongError.

        If we want to send this message as a DM, do so.

        Arguments:
        - `to`: str
        - `from_`: str
        - `body`: str
        - `dm`: [optional] bool

        Return: None
        Exceptions: AccountNotFoundError
                    TweetTooLongError
        """
        tweet = '@{0} {1}'.format(to, body)

        if from_ not in self.accounts:
            raise AccountNotFoundError()
        if len(tweet) > 140:
            raise TweetTooLongError()

        self.auth.set_access_token(*self.accounts.get(from_))
        api = tweepy.API(self.auth)
        if dm:
            api.send_direct_message(to, body)
        else:
            api.update_status(tweet)
        return


class Tweet(object):
    """
    The top level API for sending tweets
    """
    @classmethod
    def send(klass):
        dm = getattr(klass, 'DM', False)
        klass.Postie.send(klass.To, klass.From, klass.Body, dm=dm)





"""
Django utilities for use with letter.

Contact Form/View being the frist of these.
"""
from django import forms
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.views.generic.edit import FormView

try:
    from captcha.fields import ReCaptchaField
except ImportError:
    ReCaptchaField = lambda: None

u = unicode

class EmailForm(forms.Form):
    name    = forms.CharField()
    email   = forms.EmailField()
    message = forms.CharField(widget=forms.Textarea)

    def send_email(self, to):
        """
        Do work.
        """
        body = self.body()
        subject = self.subject()

        import letter

        class Message(letter.Letter):
            Postie = letter.DjangoPostman()

            From    = getattr(settings, 'DEFAULT_FROM_EMAIL', 'contact@example.com')
            To      = to
            Subject = subject
            Body    = body

        if hasattr(self, 'reply_to'):
            Message.ReplyTo = self.reply_to()

        Message.send()
        return


class ContactForm(EmailForm):
    """"
    Mailto links are awzm.
    """
    def body(self):
        return u"Contact-form from: {0}\n\n{1}".format(
            u'{0} <{1}>'.format(
                u(self.cleaned_data.get('name', '')),
                u(self.cleaned_data.get('email', ''))),
            u(self.cleaned_data.get('message', '')))

    def subject(self):
        site = Site.objects.get_current()
        return '{0} - Contact Form'.format(site.domain)


class ReCaptchaContactForm(ContactForm):
    """
    Contact form with a Captcha field.
    Requires:

    - django-recaptcha installation
    - setting RECAPTCHA_PRIVATE_KEY
    - setting RECAPTCHA_PUBLIC_KEY
    """
    captcha = ReCaptchaField()


class EmailView(FormView):
    """
    Base class for views that will send an email.

    Subclasses should specify the following properties:
    * template_name
    * form_class
    * success_url
    """
    to_addr       = getattr(settings, 'CONTACT_EMAIL',      'contact@example.com')

    def form_valid(self, form):
        """
        Praise be, someone has spammed us.
        """
        form.send_email(to=self.to_addr)
        return super(EmailView, self).form_valid(form)


class ContactView(EmailView):
    """
    Pointless form for people who don't like their email clients

    Please add a success url!
    Please add the setting CONTACT_EMAIL
    Please add the setting DEFAULT_FROM_EMAIL
    """
    template_name = 'contact.html'
    form_class    = ContactForm



"""
Commandline module interface.
"""
import argparse
import getpass
import sys

import letter

def main():
    """
    Do the things!


    Return: 0
    Exceptions:
    """
    description = 'Letter - a commandline interface'
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('--gmail', action='store_true', help='Send via Gmail', )
    args = parser.parse_args()

    to       = raw_input('To address > ')
    subject  = raw_input('Subject > ')
    body     = raw_input('Your Message > ')


    if args.gmail:
        user = fromaddr = raw_input('Gmail Address > ')
        pw   = getpass.getpass()
        postie = letter.GmailPostman(user=user, pw=pw)
    else:
        postie = letter.Postman() # Unauthorized SMTP, localhost:25
        fromaddr = raw_input('From address > ')

    class Message(letter.Letter):
        Postie     = postie

        From       = fromaddr
        To         = to
        Subject    = subject
        Body       = body

    return 0


if __name__ == '__main__':
    sys.exit(main())



"""
Unittests for the letter packate
"""
import sys
import unittest

from django.core import mail
from django.test import utils, TestCase
import ffs
import mailtools
from mock import MagicMock, patch

if sys.version_info <  (2, 7): import unittest2 as unittest

import letter


def setup_module():
    utils.setup_test_environment()

def teardown_module():
    utils.teardown_test_environment()



class BaseMailerTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_tolist(self):
        "Should be list of unicodes"
        cases = [
            ('hai',          u'hai'),
            (['hai', 'bai'], u'hai, bai')
            ]
        mailer = letter.BaseMailer()
        for case, expected in cases:
            self.assertEqual(expected, mailer.tolist(case))

    def test_sanity_check_plain(self):
        "Should raise if"
        mailer = letter.BaseMailer()
        resp = mailer.sanity_check(None, None, None, plain='this')
        self.assertEqual(None, resp)

    def test_sanity_check_html(self):
        "Should raise if"
        mailer = letter.BaseMailer()
        resp = mailer.sanity_check(None, None, None, html='this')
        self.assertEqual(None, resp)

    def test_sanity_check_plain_and_html(self):
        "Should raise if"
        mailer = letter.BaseMailer()
        resp = mailer.sanity_check(None, None, None, plain='this', html='that')
        self.assertEqual(None, resp)

    def test_sanity_check_none(self):
        "should raise"
        mailer = letter.BaseMailer()
        with self.assertRaises(letter.NoContentError):
            mailer.sanity_check(None, None, None)


class BaseSMTPMailerTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_send_no_content(self):
        "Should raise"
        mailer = letter.BaseSMTPMailer()
        with self.assertRaises(letter.NoContentError):
            mailer.send(None, None, None)




class PostmanTestCase(unittest.TestCase):

    def setUp(self):
        self.p = letter.Postman('foo', 'localhost')

    def test_find_tpl(self):
        "Find a template"
        mockpath = MagicMock(name='MockPath')
        mockpath.ls.return_value = ['that.jinja2']
        self.p.tpls = [mockpath]
        tpl = self.p._find_tpl('that')
        self.assertEqual('that.jinja2', tpl)

    def test_find_tpl_no_dir(self):
        "A dir in our path doesn't exist"
        self.p.tpls = [ffs.Path('does/not/exist/at/this/point')]
        tpl = self.p._find_tpl('that')
        self.assertEqual(None, tpl)


class DjangoPostmanTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        mail.outbox = []

    def test_send_multipart(self):
        "Should send"

        class Message(letter.Letter):
            Postie = letter.DjangoPostman()

            To      = 'larry@example.com'
            From    = 'bill@example.com'
            Subject = 'My Cool mail'

            Template = 'emails/cool_email'
            Context = {
                'href': 'http://example.com',
                'link': 'Examples!',
                }

        Message.send()



class TestLetterTestCase(unittest.TestCase):

    def test_send_no_subject(self):
        "Not failing stupidly please"
        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To       = 'larry@example.com'
            From     = 'bill@example.com'
            Template = 'hai'

        # This is essentially ensuring that we don't blow up if there is
        # no subject.
        Message.send()
        self.assertEqual(1, len(postie.send.call_args_list))

    def test_send_templated_no_context(self):
        "Not failing stupidly please"
        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To       = 'larry@example.com'
            From     = 'bill@example.com'
            Subject  = 'My Cool mail'
            Template = 'hai'

        # This is essentially testing that the call
        # to determine the template context does so
        # in a way that doesn't raise an AttributeError
        Message.send()
        self.assertEqual(1, len(postie.send.call_args_list))

    def test_send_attachments(self):
        "Should send the attachment"

        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To      = 'larry@example.com'
            From    = 'bill@example.com'
            Subject = 'My Cool mail'
            Body    = 'hai'
            Attach  = '/tmp/some.file'

        Message.send()
        args, kwargs = postie.send.call_args
        self.assertEqual(kwargs['attach'], '/tmp/some.file')

    def test_send_attachments_tuple(self):
        "Should send the attachment"

        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To      = 'larry@example.com'
            From    = 'bill@example.com'
            Subject = 'My Cool mail'
            Body    = 'hai'
            Attach  = '/tmp/some.file', '/tmp/other.file'

        Message.send()
        args, kwargs = postie.send.call_args
        self.assertEqual(kwargs['attach'], ('/tmp/some.file', '/tmp/other.file'))

    def test_send_no_attachments(self):
        "Should send the attachment"

        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To      = 'larry@example.com'
            From    = 'bill@example.com'
            Subject = 'My Cool mail'
            Body    = 'hai'

        Message.send()
        args, kwargs = postie.send.call_args
        self.assertEqual(kwargs['attach'], None)

    def test_send_attachments_template(self):
        "Should send the attachment"

        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To       = 'larry@example.com'
            From     = 'bill@example.com'
            Subject  = 'My Cool mail'
            Template = 'hai'
            Attach   = '/tmp/some.file'

        Message.send()
        args, kwargs = postie.send.call_args
        self.assertEqual(kwargs['attach'], '/tmp/some.file')

    def test_send_attachments_tuple_template(self):
        "Should send the attachment"

        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To       = 'larry@example.com'
            From     = 'bill@example.com'
            Subject  = 'My Cool mail'
            Template = 'hai'
            Attach   = '/tmp/some.file', '/tmp/other.file'

        Message.send()
        args, kwargs = postie.send.call_args
        self.assertEqual(kwargs['attach'], ('/tmp/some.file', '/tmp/other.file'))

    def test_send_no_attachments_template(self):
        "Should send the attachment"

        postie = MagicMock(name='Mock Postie')

        class Message(letter.Letter):
            Postie = postie

            To       = 'larry@example.com'
            From     = 'bill@example.com'
            Subject  = 'My Cool mail'
            Template = 'hai'

        Message.send()
        args, kwargs = postie.send.call_args
        self.assertEqual(kwargs['attach'], None)



if __name__ == '__main__':
    unittest.main()



"""
Send letters electronically.

We assume you're likely to want to send emails from templates.
Let's make that as easy as possible.
"""
from _version import __version__

import contextlib
import email
from email import encoders
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
import itertools
import mimetypes
import smtplib
import types

import ffs
from ffs.contrib import mold

__all__ = [
    '__version__',
    'Postman',
    'DjangoPostman',
    'GmailPostman',
    'setup_test_environment',
    'teardown_test_environment'
    ]

u = unicode
flatten = lambda x: [item for sublist in x for item in sublist]
stringy = lambda x: isinstance(x, types.StringTypes)
listy   = lambda x: isinstance(x, (list, tuple))

OUTBOX = []
SMTP   = None

class Error(Exception): pass
class NoTemplateError(Error): pass
class NoContentError(Error): pass

def _stringlist(*args):
    """
    Take a lists of strings or strings and flatten these into
    a list of strings.

    Arguments:
    - `*args`: "" or [""...]

    Return: [""...]
    Exceptions: None
    """
    return list(itertools.chain.from_iterable(itertools.repeat(x,1) if stringy(x) else x for x in args if x))


class Attachment(object):
    """
    A file we're attaching to an email.
    """
    def __init__(self, path):
        self.path = ffs.Path(path)

    def as_msg(self):
        """
        Convert ourself to be a message part of the appropriate
        MIME type.

        Return: MIMEBase
        Exceptions: None
        """
        # Based upon http://docs.python.org/2/library/email-examples.html
        # with minimal tweaking

        # Guess the content type based on the file's extension.  Encoding
        # will be ignored, although we should check for simple things like
        # gzip'd or compressed files.
        ctype, encoding = mimetypes.guess_type(str(self.path))
        if ctype is None or encoding is not None:
            # No guess could be made, or the file is encoded (compressed), so
            # use a generic bag-of-bits type.
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        if maintype == 'text':
            # Note: we should handle calculating the charset
            msg = MIMEText(self.path.read(), _subtype=subtype)
        elif maintype == 'image':
            fp = self.path.open('rb')
            msg = MIMEImage(fp.read(), _subtype=subtype)
            fp.close()
        elif maintype == 'audio':
            fp = self.path.open('rb')
            msg = MIMEAudio(fp.read(), _subtype=subtype)
            fp.close()
        else:
            fp = self.path.open('rb')
            msg = MIMEBase(maintype, subtype)
            msg.set_payload(fp.read())
            fp.close()
            # Encode the payload using Base64
            encoders.encode_base64(msg)

        filename = str(self.path[-1])
        msg.add_header('Content-Disposition', 'attachment', filename=filename)
        return msg


class BaseMailer(object):
    """
    Mailers either handle the construction and delivery of message
    objects once we have determined the contents etc.
    """

    def tolist(self, to):
        """
        Make sure that our addressees are a unicoded list

        Arguments:
        - `to`: str or list

        Return: [u, ...]
        Exceptions: None
        """
        return ', '.join(isinstance(to, list) and [u(x) for x in to] or [u(to)])

    def sanity_check(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None):
        """
        Sanity check the message.

        If we have PLAIN and HTML versions, send a multipart alternative
        MIME message, else send whichever we do have.

        If we have neither, raise NoContentError

        Arguments:
        - `sender`: str
        - `to`: list
        - `subject`: str
        - `plain`: str
        - `html`: str

        Return: None
        Exceptions: NoContentError
        """
        if not plain and not html:
            raise NoContentError()


class BaseSMTPMailer(BaseMailer):
    """
    Construct the message
    """

    def send(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None,
             replyto=None, attach=None):
        """
        Send the message.

        If we have PLAIN and HTML versions, send a multipart alternative
        MIME message, else send whichever we do have.

        If we have neither, raise NoContentError

        Arguments:
        - `sender`: str
        - `to`: list
        - `subject`: str
        - `plain`: str
        - `html`: str
        - `cc`: str or [str]
        - `bcc`: str or [str]
        - `replyto`: str
        - `attach`: str or [str]

        Return: None
        Exceptions: NoContentError
        """
        self.sanity_check(sender, to, subject, plain=plain, html=html)
        # Create message container - the correct MIME type is multipart/alternative.
        msg = MIMEMultipart('mixed')
        msg['Subject'] = u(subject)
        msg['From']    = u(sender)
        msg['To']      = self.tolist(to)
        if cc:
            msg['Cc']      = self.tolist(cc)

        recipients = _stringlist(to, cc, bcc)

        if replyto:
            msg.add_header('reply-to', replyto)

        # Attach parts into message container.
        # According to RFC 2046, the last part of a multipart message, in this case
        # the HTML message, is best and preferred.
        if plain:
            msg.attach(MIMEText(u(plain), 'plain'))
        if html:
            msg.attach(MIMEText(u(html), 'html'))

        # Deal with attachments.
        if attach:
            for p in _stringlist(attach):
                msg.attach(Attachment(p).as_msg())

        self.deliver(msg, recipients)


class SMTPMailer(BaseSMTPMailer):
    """
    Use SMTP to deliver our message.
    """

    def __init__(self, host, port):
        """
        Store vars
        """
        self.host = host
        self.port = port

    def deliver(self, message, to):
        """
        Deliver our message

        Arguments:
        - `message`: MIMEMultipart

        Return: None
        Exceptions: None
        """
        # Send the message via local SMTP server.
        s = smtplib.SMTP(self.host, self.port)
        # sendmail function takes 3 arguments: sender's address, recipient's address
        # and message to send - here it is sent as one string.
        s.sendmail(message['From'], to, message.as_string())
        s.quit()
        return


class SMTPAuthenticatedMailer(BaseSMTPMailer):
    """
    Use authenticated SMTP to deliver our message
    """
    def __init__(self, host, port, user, pw):
        self.host = host
        self.port = port
        self.user = user
        self.pw = pw


    def deliver(self, message, to):
        """
        Deliver our message

        Arguments:
        - `message`: MIMEMultipart

        Return: None
        Exceptions: None
        """
        # Send the message via local SMTP server.
        s = smtplib.SMTP(self.host, self.port)
        s.ehlo()
        s.starttls()
        s.login(self.user, self.pw)
        # sendmail function takes 3 arguments: sender's address, recipient's address
        # and message to send - here it is sent as one string.
        s.sendmail(message['From'], to, message.as_string())
        s.quit()
        return


class DjangoMailer(BaseMailer):
    """
    Send email using whatever is configured in our Django project's
    email settings etc etc
    """

    def send(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None,
             attach=None, replyto=None):
        """
        Send the message.

        If we have PLAIN and HTML versions, send a multipart alternative
        MIME message, else send whichever we do have.

        If we have neither, raise NoContentError

        Arguments:
        - `sender`: str
        - `to`: list
        - `subject`: str
        - `plain`: str
        - `html`: str
        - `attach`: str or iterable of str
        - `replyto`: str

        Return: None
        Exceptions: NoContentError
        """
        headers = {}
        if attach:
            raise NotImplementedError('Attachments not implemented for Django yet!')
        if replyto:
            headers['Reply-To'] = replyto

        self.sanity_check(sender, to, subject, plain=plain, html=html,
                          cc=cc, bcc=bcc)

        if not cc:
            cc = []
        if not bcc:
            bcc = []

        # This comes straight from the docs at
        # https://docs.djangoproject.com/en/dev/topics/email/
        from django.core.mail import EmailMultiAlternatives

        if not plain:
            plain = ''

        msg = EmailMultiAlternatives(u(subject), u(plain), u(sender), _stringlist(to),
                                     bcc=bcc, cc=cc, headers=headers)

        if html:
            msg.attach_alternative(u(html), "text/html")

        msg.send()
        return


class BasePostman(object):
    """
    Implement common postman-esque methods
    """

    def __init__(self, templatedir):
        """
        Set template locations
        """
        if isinstance(templatedir, (list, tuple)):
            self.tpls = [ffs.Path(t) for t in templatedir]
        else:
            self.tpls = [ffs.Path(templatedir)]
        return

    def _find_tpl(self, name, extension='.jinja2'):
        """
        Return a Path object representing the Template we're after,
        searching SELF.tpls or None

        Arguments:
        - `name`: str

        Return: Path or None
        Exceptions: None
        """
        found = None
        for loc in self.tpls:
            if not loc:
                continue
            contents = [f for f in loc.ls() if f.find(name) != -1 and f.endswith(extension)]
            if contents:
                found = contents[0]
                break
            exact = loc + (name + extension)
            if exact.is_file:
                found = exact
        return found

    def _find_tpls(self, name):
        """
        Return plain, html templates for NAME

        Arguments:
        - `name`: str

        Return: tuple
        Exceptions: None
        """
        return self._find_tpl(name, extension='.txt'), self._find_tpl(name, extension='.html')

    def _send(self, sender, to, subject, message, cc=None, bcc=None, attach=None, replyto=None):
        """
        Send a Letter (MESSAGE) from SENDER to TO, with the subject SUBJECT

        Arguments:
        - `sender`: unicode
        - `to`: unicode
        - `subject`: unicode
        - `message`: unicode
        - `cc`: str or [str]
        - `bcc`: str or [str]
        ` `replyto`: str

        Return: None
        Exceptions: None
        """
        self.mailer.send(sender, to, subject, plain=message, cc=cc, bcc=bcc, attach=attach, replyto=replyto)
        return

    send = _send

    def _sendtpl(self, sender, to, subject, cc=None, bcc=None, attach=None, replyto=None, **kwargs):
        """
        Send a Letter from SENDER to TO, with the subject SUBJECT.
        Use the current template, with KWARGS as the context.

        Arguments:
        - `sender`: unicode
        - `to`: unicode
        - `subject`: unicode
        - `cc`: str or [str]
        - `bcc`: str or [str]
        - `replyto`: str
        - `**kwargs`: objects

        Return: None
        Exceptions: None
        """
        plain, html = self.body(**kwargs)
        self.mailer.send(sender, to, subject, plain=plain, html=html, cc=cc, bcc=bcc,
                         replyto=replyto, attach=attach)
        return

    def body(self, **kwargs):
        """
        Return the plain and html versions of our contents.

        Return: tuple
        Exceptions: None
        """
        text_content, html_content = None, None
        if self.plain:
            text_content = mold.cast(self.plain, **kwargs)
        if self.html:
            html_content = mold.cast(self.html, **kwargs)
        return text_content, html_content

    @contextlib.contextmanager
    def template(self, name):
        """
        Set an active template to use with our Postman.

        This changes the call signature of send.

        Arguments:
        - `name`: str

        Return: None
        Exceptions: None
        """
        self.plain, self.html = self._find_tpls(name)
        if not self.plain:
            self.plain = self._find_tpl(name)
        try:
            self.send = self._sendtpl
            yield
        finally:
            self.plain, self.html = None, None
            self.send = self._send


class SMTPPostman(BasePostman):
    """
    The SMTP Postman is a utility class for using SMTP as
    a delivery method for our messages.
    """
    def __init__(self, templatedir=None, host='localhost', port=25):
        super(SMTPPostman, self).__init__(templatedir)
        self.mailer = SMTPMailer(host, port)


class SMTPAuthenticatedPostman(BasePostman):
    """
    The SMTP Postman is a utility class for using SMTP as
    a delivery method for our messages.
    """
    def __init__(self, templatedir=None, host='localhost', port=25, user=None, pw=None):
        super(SMTPAuthenticatedPostman, self).__init__(templatedir)
        self.mailer = SMTPAuthenticatedMailer(host, port, user, pw)


class Postman(SMTPPostman):
    """
    The Postman is your main entrypoint to sending Electronic Mail.

    Set up an SMTP mailer at HOST:PORT using TEMPLATEDIR as the place
    to look for templates.

    If BLOCKING is True, use the blocking mailer.

    Arguments:
    - `templatedir`: str
    - `host`: str
    - `port`: int

    Return: None
    Exceptions: None
    """


class DjangoPostman(BasePostman):
    """
    Postman for use with Django's mail framework.
    """
    def __init__(self):
        """
        Do Django imports...
        """
        self.html, self.plain = None, None
        from django.conf import settings
        from django.utils.functional import empty
        if settings._wrapped is empty:
            settings.configure()
        self.settings = settings
        super(DjangoPostman, self).__init__(settings.TEMPLATE_DIRS)
        self.mailer = DjangoMailer()


class GmailPostman(SMTPAuthenticatedPostman):
    """
    OK, so we're sending emails via Google's SMTP servers.

    >>> postie = GmailPostman('.', user='username', pw='password')
    """
    def __init__(self, templatedir='.', user=None, pw=None):
        super(GmailPostman, self).__init__(templatedir=templatedir,
                                           host='smtp.gmail.com',
                                           port=587,
                                           user=user,
                                           pw=pw)


class Letter(object):
    """
    An individual Letter
    """
    @classmethod
    def send(klass):
        to = klass.To
        subject = getattr(klass, 'Subject', '')

        if isinstance(to, types.StringTypes):
            to = [to]
        if getattr(klass, 'Body', None):
            klass.Postie.send(
                klass.From,
                to,
                subject,
                klass.Body,
                cc=getattr(klass, 'Cc', None),
                bcc=getattr(klass, 'Bcc', None),
                replyto=getattr(klass, 'ReplyTo', None),
                attach=getattr(klass, 'Attach', None),
                )
            return

        with klass.Postie.template(klass.Template):
            klass.Postie.send(
                klass.From,
                to,
                subject,
                cc=getattr(klass, 'Cc', None),
                bcc=getattr(klass, 'Bcc', None),
                replyto=getattr(klass, 'ReplyTo', None),
                attach=getattr(klass, 'Attach', None),
                **getattr(klass, 'Context', {})
                )

"""
Testing utilities start here.
"""
def _parse_outgoing_mail(sender, to, msgstring):
    """
    Parse an outgoing mail and put it into the OUTBOX.


    Arguments:
    - `sender`: str
    - `to`: str
    - `msgstring`: str

    Return: None
    Exceptions: None
    """
    global OUTBOX
    OUTBOX.append(email.message_from_string(msgstring))
    return

def setup_test_environment():
    """
    Set up our environment to test the sending of
    email with letter.

    We return an outbox for you, into which all emails
    will be delivered.

    Requires the Mock library.

    Return: list
    Exceptions: None
    """
    import mock
    global OUTBOX, SMTP

    SMTP = smtplib.SMTP
    mock_smtp = mock.MagicMock(name='Mock SMTP')
    mock_smtp.return_value.sendmail.side_effect = _parse_outgoing_mail
    smtplib.SMTP = mock_smtp
    return OUTBOX

def teardown_test_environment():
    """
    Tear down utilities for the testing of mail sent with letter.

    Return: None
    Exceptions: None
    """
    global OUTBOX, SMTP
    smtplib.SMTP = SMTP
    OUTBOX = []
    return



"""
Unittests for the letter.contrib.contact module
"""
import sys
import unittest

from mock import MagicMock, patch
from django.test import utils, TestCase

if sys.version_info <  (2, 7): import unittest2 as unittest

import letter
from letter.contrib import contact


def setup_module():
    utils.setup_test_environment()

def teardown_module():
    utils.teardown_test_environment()


class ContactFormTestCase(TestCase):

    def test_send_email(self):
        "Send an email"
        form = contact.ContactForm()
        form.cleaned_data = {}
        with patch.object(letter.Letter, 'send') as psend:
            with patch.object(contact.Site.objects, 'get_current') as psite:
                psite.domain = 'example.com'
                form.send_email('bill@example.com')
                psend.assert_called_once_with()



if __name__ == '__main__':
    unittest.main()



import re
import os

from distutils.core import setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()


VERSION_FILE = os.path.join(here, "letter/_version.py")
verstrline = open(VERSION_FILE, "rt").read()
VSRE = r'^__version__ = [\'"]([^\'"]*)[\'"]'
mo = re.search(VSRE,  verstrline, re.M)
if mo:
    VERSION = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in {0}".format(VERSION_FILE))

setup(
    name = "letter",
    version = VERSION,
    author = "David Miller",
    author_email = "david@deadpansincerity.com",
    url = "https://github.com/davidmiller/letter",
    description = "Send you letters for fun or profit",
    long_description = README + "\n\n" + CHANGES,
    install_requires = [
        'ffs>=0.0.7',
        'jinja2',
        ],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 2.6",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries"
        ],
    packages = ['letter', 'letter.contrib'],
    )
